;===========================================================================================
; MouseHook.asm  Fasm Flat assembler
; MouseHook afficher les coordonnes souris dans bandeau infrieur
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

  ID_STATUSBAR = 1000

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

 start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Chargement d'un menu
    ; --------------------
    invoke LoadMenu,[wc.hInstance],IDM_MOUSE
    mov [hMenu],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,ClassName,Titre,WS_OVERLAPPEDWINDOW,\
                          [WinX],[WinY],[WinW],[WinH],0,[hMenu],[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,msg,0,0,0
    or eax,eax
    jz ExitProgram
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg
    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION
        invoke UnhookWindowsHookEx,[hMouseHook]
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE
        invoke InitCommonControls

        invoke CreateStatusWindow,WS_CHILD+WS_VISIBLE,NULL,[@hWnd],ID_STATUSBAR
        mov [hWndStatus],eax

        invoke SendMessage,[hWndStatus],SB_SETPARTS,[nParts],statusWindowParts
        invoke SendMessage,[hWndStatus],SB_SETTEXT,1,ClassName

    .elseif [@uMsg] = WM_SIZE
        invoke GetClientRect,[@hWnd],rect
        mov eax,[rect.right]

        sub eax,[rect.left]
        shr eax,1
        mov ebx,statusWindowParts
        mov [ebx],eax
        mov dword[ebx+4],-1
        invoke SendMessage,[hWndStatus],SB_SETPARTS,[nParts],statusWindowParts

        invoke GetCurrentThreadId
        mov [nThreadID],eax
        invoke SetWindowsHookEx,WH_MOUSE,MouseHookProc,NULL,[nThreadID]

        invoke MoveWindow,[hWndStatus],0,0,0,0,TRUE

    .else
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret
    .endif

    mov eax,0
    ret
endp

;===========================================================================================
proc MouseHookProc @nCode, @wParam, @lParam

    .if [@nCode] > 0
        invoke CallNextHookEx,[hMouseHook],[@nCode],[@wParam],[@lParam]
    .else
        mov edx,[@lParam]              ; addr struct MOUSEHOOKSTRUCT
        mov eax,[edx]                  ; MOUSEHOOKSTRUCT.pt.x
        mov edx,[edx+4]                ; MOUSEHOOKSTRUCT.pt.y
        invoke wsprintf,szBuf,szFormat,eax,edx
        invoke SendMessage,[hWndStatus],SB_SETTEXT,0,szBuf
    .endif

    mov eax,0
    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  Titre       db ' Mouse Hook',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' AsmGges France @2006  ',0

  szFormat    db 'x: %u   y: %u',0

  align 4

  WinX        dd 0
  WinY        dd 100
  WinW        dd 300
  WinH        dd 244

  statusWindowParts dd 0,0
  nParts      dd 1

  szBuf       rb 40
  szBuffer    rb 530

  hWnd        dd ?
  hMouseHook  dd ?
  nLastError  dd ?
  nThreadID   dd ?
  hMenu       dd ?
  hWndStatus  dd ?

  wc          WNDCLASS  CS_HREDRAW+CS_VREDRAW, WindowProc, 0, 0, 0, 0, 0, COLOR_WINDOW+1,\
                                                                            0, ClassName
  msg         MSG
  icex        INITCOMMONCONTROLSEX
  rect        RECT

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL',\
          comctl32,'COMCTL32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'
  include 'apia\comctl32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons

  IDI_FRANCE = 17
  IDM_MOUSE  = 100

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

;===========================================================================================

